@@ -4,7 +4,9 @@ from .arrayfuncs import assure_numpy_array, assure_pandas_df, check_1d, warn_if_
 from .warnings import ApproximationWarning
 from ._utils import class_name_from_object, assure_list_of_strings, assure_list_values_allowed
 from .tree import TreePathFinder
+from .plots import plot_distributions_of_feature
 
 __all__ = ['NotFittedError', 'DimensionalityError', 'UnsupportedModelError', 'Scorer', 'assure_numpy_array', 'assure_pandas_df', 'check_1d',
            'ApproximationWarning', 'class_name_from_object', 'get_scorers', 'assure_list_of_strings',
-           'assure_list_values_allowed', 'warn_if_missing','TreePathFinder', 'check_numeric_dtypes']
+           'assure_list_values_allowed', 'warn_if_missing','TreePathFinder', 'check_numeric_dtypes',
+           'plot_distributions_of_feature']
